
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "QRMatrix.h"

#include "QRInformation.h"
#include "QRReadSolomon.h"

///////////////////////////////////////////////////////////////////////////////
///		CQRMatrix̒`
///////////////////////////////////////////////////////////////////////////////
CQRMatrix::CQRMatrix()
{
	m_iModelTypeIndex = -1;
	m_iModuleNumber = 0;
	m_ppucQRMatrix = NULL;
	m_ppucMaskMatrix = NULL;

	m_iDataBitNumber = 0;
	m_iDataSizeNumber = 0;
	m_pucData = NULL;

	m_iErrorSizeNumber = 0;
	m_pucError = NULL;

	m_iQRMaskPattern = -1;
}

CQRMatrix::~CQRMatrix()
{
	ReleaseAll();
}

///////////////////////////////////////////////////////////
///		Xgɕf[^Zbg
///////////////////////////////////////////////////////////
bool CQRMatrix::AddQRCharacters( const char * pcszString , int iCharacterType )
{
	CQRDataElement qrElement;

	bool bDataAdded = false;

	//Pȏ̃f[^o^Ăꍇ
	if( m_listQRData.begin() != m_listQRData.end() )
	{
		list<CQRDataElement>::iterator pQRElement = m_listQRData.end();
		pQRElement--;

		//Ō̃f[^[hƌ݂̃f[^[hvĂꍇɂ͍Ō̃Xg
		//ǉ
		bDataAdded = pQRElement->AddData( pcszString , (int)strlen(pcszString) , iCharacterType );
	}

	if( !bDataAdded )
	{
		qrElement.SetData( pcszString , (int)strlen( pcszString ) , iCharacterType );
		m_listQRData.push_back( qrElement );
	}
	

	return true;
}

///////////////////////////////////////////////////////////
///		Sf[^폜
///////////////////////////////////////////////////////////
void CQRMatrix::ReleaseAll()
{
	ReleaseQRMatrix();
	ReleaseMaskMatrix();
	ReleaseData();
	ReleaseError();

	m_iModuleNumber = 0;
	m_iModelTypeIndex = -1;
}

///////////////////////////////////////////////////////////
///		QR̃f[^폜
///////////////////////////////////////////////////////////
void CQRMatrix::ReleaseQRMatrix()
{
	if( m_ppucQRMatrix != NULL )
	{
		for( int i=0 ; i<m_iModuleNumber ; i++ )
		{
			delete[] m_ppucQRMatrix[i];
		}
		delete[] m_ppucQRMatrix;
	}
	m_ppucQRMatrix = NULL;
}

///////////////////////////////////////////////////////////
///		}XÑf[^폜
///////////////////////////////////////////////////////////
void CQRMatrix::ReleaseMaskMatrix()
{
	if( m_ppucMaskMatrix )
	{
		for( int i=0 ; i<m_iModuleNumber ; i++ )
		{
			delete[] m_ppucMaskMatrix[i];
		}
		delete[] m_ppucMaskMatrix;
	}

	m_ppucMaskMatrix = NULL;
}

void CQRMatrix::ReleaseData()
{
	if( m_pucData )
	{
		delete[] m_pucData;
	}
	m_pucData = NULL;
	m_iDataSizeNumber = 0;
}

void CQRMatrix::ReleaseError()
{
	if( m_pucError )
	{
		delete[] m_pucError;
	}
	m_pucError = NULL;
	m_iErrorSizeNumber = 0;
}

///////////////////////////////////////////////////////////
///		ZbgĂf[^ɂpq}gbNX쐬
///		F
///			iCollectLevel	F	x
///									COLLECTLEVEL_L
///									COLLECTLEVEL_M
///									COLLECTLEVEL_Q
///									COLLECTLEVEL_H
///								̂ꂩw肷
///		߂lF
///			==falseF쐬Ɏsif[^傫Ȃǁj
///			==trueF쐬ɐ
///////////////////////////////////////////////////////////
bool CQRMatrix::MakeQRMatrix( int iCollectLevel , int & iErrorType )
{
	//ݐݒ肳ĂQRf[^̈폜
	ReleaseAll();

	bool bRet = true;
	iErrorType = QRERR_NOERROR;

	//^ԏݒ肷
	bRet = SetTypeNumber( iCollectLevel , iErrorType );

	//f[^rbg쐬
	if( bRet )
	{
		bRet = CreateDataBits( iErrorType );
	}

	//R[h̍쐬
	if( bRet )
	{
		bRet = CreateErrorCollectData( iErrorType );
	}

	//Zbgꂽ^ԍAW[z
	if( bRet )
	{
		bRet = InitModuleArea( iErrorType );
	}

	//ʒuop^[̃Zbg
	if( bRet )
	{
		bRet = SetPositionDitectPattern( iErrorType );
	}

	//ʒu킹p^[̃Zbg
	if( bRet )
	{
		bRet = SetPositionCheckPattern( iErrorType );
	}

	//^C~Op^[̃Zbg
	if( bRet )
	{
		bRet = SetTimingPattern( iErrorType );
	}

	//`̉o
	if( bRet )
	{
		bRet = SetTypeInformation( true , iErrorType );
	}

	//^ԏ
	if( bRet )
	{
		bRet = SetModelInformation( iErrorType );
	}

	//}XNp^[K̈擾
	if( bRet )
	{
		bRet = CreateMaskPatternArea( iErrorType );
	}

	//f[^rbg̏o
	if( bRet )
	{
		bRet = OutputDataBits( iErrorType );
	}

	//}XNp^[̌
	if( bRet )
	{
		bRet = DecideMaskPattern( iErrorType );
	}

	//`̏o
	if( bRet )
	{
		bRet = SetTypeInformation( false, iErrorType );
	}

	return bRet;
}

///////////////////////////////////////////////////////////
///		o^Ăf[^f[^쐬
///////////////////////////////////////////////////////////
bool CQRMatrix::CreateDataBits( int & iErrorType )
{
	bool bRet = true;

	//f[^̃[X
	ReleaseData();

	list<CQRDataElement>::iterator pDataElement;
	for( pDataElement = m_listQRData.begin()
		;pDataElement != m_listQRData.end() && bRet 
		;pDataElement ++ )
	{
		//f[^[h̏o
		unsigned char cModel = 0x00;
		switch( pDataElement->m_iDataMode )
		{
		case QRCHARACTERTYPE_NUMERIC:
			bRet = AddDataBits_Numeric( pDataElement->m_pszData , pDataElement->m_iDataLength , iErrorType);
			break;

		case QRCHARACTERTYPE_ALPHABET:
			bRet = AddDataBits_Alphabet( pDataElement->m_pszData , pDataElement->m_iDataLength , iErrorType );
			break;

		case QRCHARACTERTYPE_ASCII:
			bRet = AddDataBits_Ascii( pDataElement->m_pszData , pDataElement->m_iDataLength , iErrorType );
			break;

		case QRCHARACTERTYPE_KANJI:
			bRet = AddDataBits_Kanji( pDataElement->m_pszData , pDataElement->m_iDataLength , iErrorType );
			break;
		}

	}


	//I[o
	if( bRet )
	{
		bRet = AddDataBits( (unsigned char)0x00 , 4 , iErrorType);
	}

	if( bRet )
	{
		//f[^oCgI[܂ł0Ŗ߂
		for( ; m_iDataBitNumber < (c_typeMatrix + m_iModelTypeIndex)->m_iDataBitNumber &&
			   m_iDataBitNumber % 8 != 0 && bRet
			 ; )
		{
			bRet = AddDataBits( (unsigned char)0x00 , 1 , iErrorType );
		}

		bool bFirstByte = true;
		for( ; m_iDataBitNumber < (c_typeMatrix + m_iModelTypeIndex)->m_iDataBitNumber && bRet
			 ;  )
		{
			if( bFirstByte )
			{
				bRet = AddDataBits( (unsigned char)0xec , 8 , iErrorType );
			}
			else
			{
				bRet = AddDataBits( (unsigned char)0x11 , 8 , iErrorType );
			}
			bFirstByte = !bFirstByte;
		}
	}

	return bRet;
}

///////////////////////////////////////////////////////////
///		o^ς݂̃f[^Xgf[^̃rbg擾
///		pq̌^ԂɂATCYقȂ邽߁Ae^Ԏ̃f[^TCY
///		Zbg
///		F
///			iDataBit1_9		(OUT)	1^`9^̃f[^rbg
///			iDataBit10_26	(OUT)	10^`26^̃f[^rbg
///			iDataBit27_40	(OUT)	27^`40^̃f[^rbg
///////////////////////////////////////////////////////////
bool CQRMatrix::GetDataBitNumber( int & iDataBit1_9 , int & iDataBit10_26 , int & iDataBit27_40 )
{
	bool bRet = true;

	//
	iDataBit1_9 = iDataBit10_26 = iDataBit27_40 = 0;
	//^Ɋ֌WȂf[^rbg
	int iDataBits = 0;

	list<CQRDataElement>::iterator pDataElement;
	for( pDataElement = m_listQRData.begin()
		; pDataElement != m_listQRData.end()
		; pDataElement++ )
	{
		//f[^wq
		iDataBits += 4;

		switch( pDataElement->m_iDataMode )
		{
		//[h̏ꍇ
		case QRCHARACTERTYPE_NUMERIC:

			//f[^TCYo͗p
			iDataBit1_9   += 10;
			iDataBit10_26 += 12;
			iDataBit27_40 += 14;

			//f[^{̏o͗p
			iDataBits += ( pDataElement->m_iDataLength / 3 ) * 10;
			if( pDataElement->m_iDataLength % 3 == 1 )
			{
				iDataBits += 4;
			}
			else if( pDataElement->m_iDataLength % 3 == 2 )
			{
				iDataBits += 7;
			}
			break;

		//At@xbg[h̏ꍇ
		case QRCHARACTERTYPE_ALPHABET:

			//f[^TCYo͗p
			iDataBit1_9   += 9;
			iDataBit10_26 += 11;
			iDataBit27_40 += 13;

			//f[^{̏o͗p
			iDataBits += ( pDataElement->m_iDataLength / 2 ) * 11;
			if( pDataElement->m_iDataLength % 2 ) 
			{
				iDataBits += 6;
			}

			break;

		//ASCII
		case QRCHARACTERTYPE_ASCII:

			//f[^TCYo͗p
			iDataBit1_9 += 8;
			iDataBit10_26 += 16;
			iDataBit27_40 += 16;

			iDataBits += pDataElement->m_iDataLength * 8;
			break;

		//p
		case QRCHARACTERTYPE_KANJI:

			//f[^TCYo͗p
			iDataBit1_9   += 8;
			iDataBit10_26 += 10;
			iDataBit27_40 += 12;

			//f[^o͗p
			iDataBits += (pDataElement->m_iDataLength / 2) * 13;

			break;

		default:
			return false;
		}
	}

	//I[[hwq
	iDataBits += 4;

	iDataBit1_9 += iDataBits;
	iDataBit10_26 += iDataBits;
	iDataBit27_40 += iDataBits;

	return bRet;
}

///////////////////////////////////////////////////////////
///		f[^̃Zbg
///////////////////////////////////////////////////////////
bool CQRMatrix::AddDataBits_Numeric( const char * pcszData , int iDataLength , int & iErrorType )
{
	bool bRet = true;

	//f[^݂̂ǂ̃`FbN
	for( int i=0 ; i<iDataLength ; i++ )
	{
		char cTemp = pcszData[i];

		//łȂꍇ͑߂
		if( !isdigit( cTemp ) )
		{
			iErrorType = QRERR_DATAISNOT_NUMERIC;
			return false;
		}

	}

	//̎wqo
	bRet = AddDataBits( (unsigned char)0x01 , 4 , iErrorType );

	//wq̏o
	short sOutputNumber = (short)iDataLength;

	int iModelNumber = (c_typeMatrix + m_iModelTypeIndex)->m_iModelNumber;

	//P`X^̏ꍇiPOrbgj
	if( iModelNumber <= 9 )
	{
		bRet = AddDataBits( (unsigned char)( (sOutputNumber >> 8) & 0x03 ) , 2 , iErrorType );
	}

	//PO`QU^̏ꍇiPQrbgj
	else if(iModelNumber <= 26 )
	{
		bRet = AddDataBits( (unsigned char)( (sOutputNumber >> 8) & 0x0f ) , 4 , iErrorType );
	}

	//QV`SO^̏ꍇiPSrbgj
	else
	{
		bRet = AddDataBits( (unsigned char)( (sOutputNumber >> 8) & 0x3f ) , 6 , iErrorType );
	}

	//ʂWrbgo
	if( bRet )
	{
		bRet = AddDataBits( (unsigned char)( sOutputNumber & 0xff ) , 8 , iErrorType );
	}

	for( int i=0 ; i<iDataLength && bRet ; i+= 3 )
	{
		char szTemp[4];
		int iIndex = 0;

		for( int j=i ; j<iDataLength && iIndex < 3 ; j++,iIndex++ )
		{
			szTemp[iIndex] = pcszData[j];
		}
		szTemp[iIndex] = '\0';

		short sTemp = (short)atoi( szTemp );
		size_t sizeLength = strlen( szTemp );

		//P̂
		if( sizeLength == 1 )
		{
			bRet = AddDataBits( (unsigned char)(sTemp & 0x0f) , 4 , iErrorType );
		}
		//Q
		else if( sizeLength == 2 )
		{
			bRet = AddDataBits( (unsigned char)(sTemp & 0x7f) , 7 , iErrorType );
		}
		//R
		else if( sizeLength == 3 )
		{
			bRet = AddDataBits( (unsigned char)((sTemp >> 8) & 0x03) , 2 , iErrorType );
			if( bRet )
			{
				bRet = AddDataBits( (unsigned char)( sTemp & 0xff ) , 8 , iErrorType );
			}
		}
	}

	return bRet;
}

///////////////////////////////////////////////////////////
///		At@xbgf[^̃Zbg
///////////////////////////////////////////////////////////
bool CQRMatrix::AddDataBits_Alphabet( const char * pcszData , int iDataLength , int & iErrorType )
{
	bool bRet = true;

	//f[^p݂̂ǂ`FbN
	for( int i=0 ; i<iDataLength ; i++ )
	{
		char cTemp = pcszData[i];
		bRet = false;
		for( int j = 0 ; cszQrAlphabets[j] ; j++ )
		{
			if( cTemp == cszQrAlphabets[j] )
			{
				bRet = true;
				break;
			}
		}

		if( !bRet )
		{
			return false;
		}
	}

	//At@xbg̎wqo
	bRet = AddDataBits( (unsigned char)0x02 , 4 , iErrorType );

	//̏o
	unsigned short usLength = (unsigned short)iDataLength;
	int iModelTypeNumber = (c_typeMatrix + m_iModelTypeIndex)->m_iModelNumber;
	
	//P`X^
	if( iModelTypeNumber <= 9 )
	{
		bRet = AddDataBits( (unsigned char)(( usLength >> 8 ) & 0x01) , 1 , iErrorType );
	}
	//PO`QU^
	else if( iModelTypeNumber <= 26 )
	{
		bRet = AddDataBits( (unsigned char)(( usLength >> 8 ) & 0x07) , 3 , iErrorType );
	}
	//QV`SO^
	else
	{
		bRet = AddDataBits( (unsigned char)(( usLength >> 8 ) & 0x1f) , 5 , iErrorType );
	}

	if( bRet )
	{
		bRet = AddDataBits( (unsigned char)(usLength & 0xff) , 8 , iErrorType );
	}

	//QPPrbgɕϊ
	for( int i=0 ; i<iDataLength && bRet ; i+=2 )
	{
		unsigned short usChar = 0;
		int iChar;

		for( iChar = 0; iChar < 2 && i+iChar<iDataLength ; iChar++ )
		{
			usChar *= 45;

			for( int j=0 ; cszQrAlphabets[j] ; j++ )
			{
				if( cszQrAlphabets[j] == pcszData[i+iChar] )
				{
					usChar += (unsigned char)j;
				}
			}
		}

		//P݂̂̏o͂̏ꍇiUbitój
		if( iChar == 1 )
		{
			bRet = AddDataBits( (unsigned char)(usChar & 0x3f) , 6 , iErrorType );
		}
		//Q̏o͂̏ꍇiPPbitój
		else if( iChar == 2 )
		{
			bRet = AddDataBits( (unsigned char)((usChar >> 8) & 0x07) , 3 , iErrorType );
			if( bRet )
			{
				bRet = AddDataBits( (unsigned char)(usChar & 0xff) , 8 , iErrorType );
			}
		}
	}

	return bRet;
}

///////////////////////////////////////////////////////////
///		AXL[f[^̃Zbg
///////////////////////////////////////////////////////////
bool CQRMatrix::AddDataBits_Ascii( const char * pcszData , int iDataLength , int & iErrorType )
{
	bool bRet = true;

	//AXL[̎wqo
	bRet = AddDataBits( (unsigned char)0x04 , 4 , iErrorType );
	
	if( bRet )
	{
		//f[^̏o
		int iModelNumber = (c_typeMatrix + m_iModelTypeIndex)->m_iModelNumber;

		//P^`X^
		if( iModelNumber <= 9 )
		{
			bRet = AddDataBits( (unsigned char)iDataLength , 8 , iErrorType );
		}
		//PO^`SO^
		else
		{
			bRet = AddDataBits( (unsigned short)iDataLength , iErrorType );
		}
	}

	for( int i=0 ; i<iDataLength && bRet ; i++ )
	{
		unsigned char ucTemp = (unsigned char)pcszData[i];
		bRet = AddDataBits( ucTemp , 8 , iErrorType );
	}

	return bRet;
}

///////////////////////////////////////////////////////////
///		f[^̃Zbg
///////////////////////////////////////////////////////////
bool CQRMatrix::AddDataBits_Kanji( const char * pcszData , int iDataLength , int & iErrorType )
{
	bool bRet = true;

	//ǂ̃`FbN
	for( int i=0 ; i<iDataLength ; i+=2 )
	{
		unsigned short usTemp = (((((unsigned short)pcszData[i]) << 8) & 0xff00) |
									((unsigned short)pcszData[i+1] & 0x00ff) );

		//0x8140`0x9ffc
		if( !(0x8140 <= usTemp && usTemp <= 0x9ffc || 
			  0xe040 <= usTemp && usTemp <= 0xebbf) )
		{
			iErrorType = QRERR_DATAISNOT_KANJI;
			return false;
		}

	}

	//[h
	bRet = AddDataBits( (unsigned char)0x08 , 4 , iErrorType );
	if( !bRet )
	{
		return bRet;
	}

	//̏o
	short sLength = (short)(iDataLength / 2);
	int iModelNumber = (c_typeMatrix + m_iModelTypeIndex)->m_iModelNumber;

	//P`X^
	if( iModelNumber <= 9 )
	{
		bRet = AddDataBits( (unsigned char)( sLength & 0xff ) , 8 , iErrorType );
	}
	else
	{
		//PO`QU^
		if( iModelNumber <= 26 )
		{
			bRet = AddDataBits( (unsigned char)((sLength >> 8) & 0x03) , 2 , iErrorType );
		}
		//QV`SO^
		else
		{
			bRet = AddDataBits( (unsigned char)((sLength >> 8) & 0x0f) , 4 , iErrorType );
		}

		if( bRet )
		{
			bRet = AddDataBits( (unsigned char)(sLength & 0xff) , 8 , iErrorType );
		}
	}

	//̏o
	for( int i=0 ; i<iDataLength && bRet ; i+=2 )
	{
		unsigned short usCharacter = (((((unsigned short)pcszData[i]) << 8) & 0xff00) |
										((unsigned short)pcszData[i+1] & 0x00ff) );

		//0x8140`0x9ffc
		if( 0x8140 <= usCharacter && usCharacter <= 0x9ffc )
		{
			//0x8140
			usCharacter -= 0x8140;
		}
		//0xe040`0xebbf
		else
		{
			//0xc140
			usCharacter -= 0xc140;
		}
		usCharacter = (((usCharacter >> 8) & 0xff) * 0xc0)	//ʃoCgC0悶
					+  ((usCharacter) & 0xff);				//ʃoCg𑫂

		//f[^̏o͏
		bRet = AddDataBits( ((unsigned char)(usCharacter >> 8) & 0x1f) , 5 , iErrorType );
		if( bRet )
		{
			bRet = AddDataBits( (unsigned char)(usCharacter & 0xff) , 8 , iErrorType );
		}
	}

	return bRet;
}

///////////////////////////////////////////////////////////
///		f[^Ƀrbgǉ
///////////////////////////////////////////////////////////
bool CQRMatrix::AddDataBits( unsigned char ucData , int iBitNumber , int & iErrorType )
{
	bool bRet = true;

	if( iBitNumber <= 0 || iBitNumber > 8 )
	{
		iErrorType = QRERR_WRONGDIRECT_BIT;
		return false;
	}

	int iBitLast = m_iDataBitNumber + iBitNumber;
	//KvoCgTCY
	int iByteNow = (m_iDataBitNumber + 7) / 8;
	int iByteNeed = (iBitLast + 7) / 8;

	//ǉ̃f[^̈悪Kvȏꍇ
	if( iByteNeed > m_iDataSizeNumber )
	{
		//128oCg
		m_iDataSizeNumber += 128;
		unsigned char * puszTemp = new unsigned char[ m_iDataSizeNumber ];
		memset( puszTemp , 0 , m_iDataSizeNumber );

		if( m_pucData )
		{
#ifdef WIN32
			memcpy_s( puszTemp , m_iDataSizeNumber , m_pucData , iByteNow );
#else
			memcpy( puszTemp , m_pucData , iByteNow );
#endif
			delete[] m_pucData;
		}

		m_pucData = puszTemp;
	}

	//Zbgf[^̃}XN
	unsigned char ucSetMask = GetBitMask( iBitNumber - 1 );

	//f[^̒ǉ
	for( int i=0 ; i < iBitNumber ; i++ )
	{
		//biton̏ꍇ
		if( ucSetMask & ucData )
		{
			int iBitToSet = m_iDataBitNumber + i;

			//Zbgς݃f[^̎̃f[^̃}XN
			unsigned char ucDataMask = GetBitMask( 7 - (iBitToSet % 8) );

			//Yf[^bitonɂ
			int iByteNow = iBitToSet / 8;

			m_pucData[iByteNow] |= ucDataMask;
			
		}
		ucSetMask = (ucSetMask >> 1) & 0x7f;
	}
	
	m_iDataBitNumber += iBitNumber;

	return bRet;
}

///////////////////////////////////////////////////////////
///		QoCgf[^o͂
///////////////////////////////////////////////////////////
bool CQRMatrix::AddDataBits( unsigned short usData , int & iErrorType )
{
	bool bRet = true;

	bRet = AddDataBits( (unsigned char)((usData >> 8 ) & 0xff) , 8 , iErrorType );
	if( bRet )
	{
		bRet = AddDataBits( (unsigned char)(usData & 0xff) , 8 , iErrorType );
	}
	return bRet;
}

///////////////////////////////////////////////////////////
///		w肳ꂽrbgԍ̃rbg}XN擾
///////////////////////////////////////////////////////////
unsigned char CQRMatrix::GetBitMask( int iBitNumber )
{
	unsigned char ucMask = 0x01 << iBitNumber;

	return ucMask;
}

///////////////////////////////////////////////////////////
///		f[^̃oCgƌx
///		QŘ^i1^`40^j肷
///////////////////////////////////////////////////////////
bool CQRMatrix::SetTypeNumber(int iCollectLevel , int & iErrorType )
{
	int iByteNum = 0;

	//w肳ꂽxs̏ꍇ
	if( iCollectLevel != COLLECTLEVEL_L && iCollectLevel != COLLECTLEVEL_M &&
		iCollectLevel != COLLECTLEVEL_Q && iCollectLevel != COLLECTLEVEL_H )
	{
		iErrorType = QRERR_WRONG_COLLECTTYPE;
		return false;
	}

	bool bRet = false;

	int iDataBit1_9 , iDataBit10_26 , iDataBit27_40;
	if( !GetDataBitNumber( iDataBit1_9 , iDataBit10_26 , iDataBit27_40 ) )
	{
		iErrorType = QRERR_DATABITCOUNT_FAILURE;
		return false;
	}

	//^ԍ\̂猟
	for( int i=0 ; (c_typeMatrix+i)->m_iModelNumber > 0 ; i++ )
	{
		//^ɂăTCYقȂ邽߁Ã݂`FbN^ɍ킹rbgTCY擾
		int iDataBitNumber = iDataBit1_9;
		if( (c_typeMatrix + i)->m_iModelNumber >= 27 )
		{
			iDataBitNumber = iDataBit27_40;
		}
		else if( (c_typeMatrix + i)->m_iModelNumber >= 10 )
		{
			iDataBitNumber = iDataBit10_26;
		}

		if( (c_typeMatrix+i)->m_iCollectLevel == iCollectLevel &&
			(c_typeMatrix+i)->m_iDataBitNumber >= iDataBitNumber )
		{
			m_iModelTypeIndex = i;

			bRet = true;
			break;
		}
	}

	//G[ꍇ
	if( !bRet )
	{
		iErrorType = QRERR_DATASIZE_LARGE;
	}

	return bRet;
}

///////////////////////////////////////////////////////////
///		R[h̍쐬
///////////////////////////////////////////////////////////
bool CQRMatrix::CreateErrorCollectData( int & iErrorType )
{
	bool bRet = true;

	ReleaseError();

	//f[^R[hꐔirbgj
	int iErrorBit = (c_typeMatrix + m_iModelTypeIndex)->m_iTotalBits 
				  - (c_typeMatrix + m_iModelTypeIndex)->m_iDataBitNumber;
	m_iErrorSizeNumber = iErrorBit / 8;
	m_pucError = new unsigned char[m_iErrorSizeNumber];

	int iFirstCount , iSecondCount;
	int iFirstDataNumber , iSecondDataNumber;

	int iSeparateCount = (c_typeMatrix + m_iModelTypeIndex)->m_iRSBlockNumber;
	int iTotalNumber = (c_typeMatrix + m_iModelTypeIndex)->m_iDataBitNumber / 8;

	SeparateTotalNumber( iSeparateCount , iTotalNumber , iFirstCount , iFirstDataNumber , iSecondCount , iSecondDataNumber );

	int iDataIndex = 0;
	int iErrorIndex = 0;
	int iErrorNumber = ( (c_typeMatrix + m_iModelTypeIndex)->m_iTotalBits
						-(c_typeMatrix + m_iModelTypeIndex)->m_iDataBitNumber) / 8 / iSeparateCount;

	//eubNƂɃ[h\ɂ镄s
	for( int i=0 ; i<iSeparateCount ; i++ )
	{
		int iDataNumber = iSecondDataNumber;
		if( i < iFirstCount )
		{
			iDataNumber = iFirstDataNumber;
		}

		CQRReadSolomon qrReadSolomon;

		qrReadSolomon.CodeTo( m_pucData + iDataIndex , iDataNumber , m_pucError + iErrorIndex , iErrorNumber );

		iDataIndex += iDataNumber;
		iErrorIndex += iErrorNumber;
	}
	return bRet;
}

///////////////////////////////////////////////////////////
///		f[^̕
///		[ꍇAiFirstCountiSecondCountɕ
///		ŕł悤ɒ
///		RSubNɑ΂f[^vẐɎgp
///////////////////////////////////////////////////////////
bool CQRMatrix::SeparateTotalNumber( int iSeparateCount , int iTotalNumber
									, int & iFirstCount , int & iFirstNumber
									, int & iSecondCount , int & iSecondNumber )
{
	iFirstNumber = iTotalNumber / iSeparateCount;
	
	iSecondCount = iTotalNumber - iFirstNumber * iSeparateCount;
	if( iSecondCount == 0 )
	{
		iSecondNumber = 0;
	}
	else
	{
		iSecondNumber = iFirstNumber + 1;
	}

	iFirstCount = iSeparateCount - iSecondCount;

	return true;
}

///////////////////////////////////////////////////////////
///		ݒ肳ꂽ^ԍgpāAW[̈
///		mۂA
///////////////////////////////////////////////////////////
bool CQRMatrix::InitModuleArea( int & iErrorType )
{
	//^ԍ烂W[vZ
	m_iModuleNumber = 17 + (c_typeMatrix + m_iModelTypeIndex)->m_iModelNumber * 4;

	//W[f[^̗̈mۂƏ
	m_ppucQRMatrix = new char*[m_iModuleNumber];
	//}XNf[^̗̈mۂƏ
	m_ppucMaskMatrix = new char*[m_iModuleNumber];

	for( int i=0 ; i<m_iModuleNumber ; i++ )
	{
		m_ppucQRMatrix[i]   = new char[m_iModuleNumber];
		m_ppucMaskMatrix[i] = new char[m_iModuleNumber];

		for( int j=0 ; j<m_iModuleNumber ; j++ )
		{
			m_ppucQRMatrix[i][j] = (char)QR_UNKNOWN;
			m_ppucMaskMatrix[i][j] = (char)QR_UNKNOWN;
		}
	}

	return true;
}

///////////////////////////////////////////////////////////
///		ʒuop^[̃Zbg
///		ʒuop^[̕p^[ɃZbg
///////////////////////////////////////////////////////////
bool CQRMatrix::SetPositionDitectPattern( int & iErrorType )
{
	bool bRet = true;

	char cPatterns[9][10] 
				= {"000000000"
				  ,"011111110"
				  ,"010000010"
				  ,"010111010"
				  ,"010111010"
				  ,"010111010"
				  ,"010000010"
				  ,"011111110"
				  ,"000000000"};

	//ꌟop^[ʒu
	int iStartPosX[3] = { -1 , m_iModuleNumber - 8 , -1 };
	int iStartPosY[3] = { -1 , -1                  , m_iModuleNumber - 8 };

	for( int i=0 ; i<3 ; i++ )
	{
		//p^[̏o
		for( int j=0 ; j<9 ; j++ )
		{
			for( int k=0 ; k<9 ; k++ )
			{
				int iModuleType = QR_POSITIONDITECT_LIGHT;

				if( cPatterns[j][k] == '1' )
				{
					iModuleType = QR_POSITIONDITECT_DARK;
				}

				int x = iStartPosX[i] + j;
				int y = iStartPosY[i] + k;

				SetModulePoint( x , y , iModuleType );
			}
		}

	}

	return bRet;
}

///////////////////////////////////////////////////////////
///		ʒu킹p^[̃Zbg
///////////////////////////////////////////////////////////
bool CQRMatrix::SetPositionCheckPattern( int & iErrorType )
{
	bool bRet = true;

	char cPatterns[5][6]
					={"11111"
					 ,"10001"
					 ,"10101"
					 ,"10001"
					 ,"11111"};

	//^Ԃɂʒu킹p^[̏c[v
	int iModelNumber = (c_typeMatrix + m_iModelTypeIndex)->m_iModelNumber;
	for( int i=0 ; c_sPositionChecks[iModelNumber-1][i] ; i++ )
	{
		for( int j=0 ; j<c_sPositionChecks[iModelNumber-1][j] ; j++ )
		{
			//ʒuop^[Ƃ̔肪Ȃǂ`FbN
			bool bOutput = true;

			//
			int x = c_sPositionChecks[iModelNumber-1][i] - 2;
			int y = c_sPositionChecks[iModelNumber-1][j] - 2;
			if( GetModulePoint( x , y ) != QR_UNKNOWN )
			{
				bOutput = false;
			}

			//E
			x += 4;
			if( GetModulePoint( x , y ) != QR_UNKNOWN )
			{
				bOutput = false;
			}

			//E
			y += 4;
			if( GetModulePoint( x , y ) != QR_UNKNOWN )
			{
				bOutput = false;
			}

			//
			x -= 4;
			if( GetModulePoint( x , y ) != QR_UNKNOWN )
			{
				bOutput = false;
			}

			if( bOutput )
			{
				//eʒu킹p^[̏o͗p{bNX[v
				for( int k = 0 ; k < 5 ; k++ )
				{
					for( int l = 0 ; l < 5 ; l++ )
					{
						int x = c_sPositionChecks[iModelNumber-1][i] + k - 2;
						int y = c_sPositionChecks[iModelNumber-1][j] + l - 2;

						//̔
						int iModuleType = QR_POSITIONCHECK_LIGHT;
						if( cPatterns[k][l] == '1' )
						{
							iModuleType = QR_POSITIONCHECK_DARK;
						}
						SetModulePoint( x , y , iModuleType );

					}
				}
			}
		}
	}

	return bRet;
}

///////////////////////////////////////////////////////////
///		^C~Op^[̃Zbg
///////////////////////////////////////////////////////////
bool CQRMatrix::SetTimingPattern( int & iErrorType )
{
	bool bRet = true;

	for( int i=0 ; i<m_iModuleNumber ; i++ )
	{
		int iModuleType = ((i%2)?(QR_TIMING_LIGHT):(QR_TIMING_DARK));

		int x = i;
		int y = 6;
		if( GetModulePoint( x , y ) == QR_UNKNOWN )
		{
			SetModulePoint( x , y , iModuleType );
		}

		x = 6;
		y = i;
		if( GetModulePoint( x , y ) == QR_UNKNOWN )
		{
			SetModulePoint( x , y , iModuleType );
		}

	}

	return bRet;
}

///////////////////////////////////////////////////////////
///		`o
///		F
///			bReserveOnly	F	truew肷ƃf[^̈
///								̗\siׂLIGHTœhԂj
///////////////////////////////////////////////////////////
bool CQRMatrix::SetTypeInformation( bool bReserveOnly , int & iErrorType )
{
	//\݂̂̏ꍇ
	if( bReserveOnly )
	{
		//̉
		for( int i=0 ; i<9 ; i++ )
		{
			int x = i;
			int y = 8;
			if( GetModulePoint( x , y ) == QR_UNKNOWN )
			{
				SetModulePoint( x , y , QR_TYPE_LIGHT );
			}
		}

		//̏c
		for( int i = 0 ; i < 8 ; i++ )
		{
			int x = 8;
			int y = i;
			if( GetModulePoint( x , y ) == QR_UNKNOWN )
			{
				SetModulePoint( x , y , QR_TYPE_LIGHT );
			}
		}

		//Ẻ
		for( int i=0 ; i<8 ; i++ )
		{
			int x = m_iModuleNumber - 8 + i;
			int y = 8;
			if( GetModulePoint( x , y ) == QR_UNKNOWN )
			{
				SetModulePoint( x , y , QR_TYPE_LIGHT );
			}
		}

		//̏c
		for( int i=0 ; i<8 ; i++ )
		{
			int x = 8;
			int y = m_iModuleNumber - 8 + i;
			if( GetModulePoint( x , y ) == QR_UNKNOWN )
			{
				SetModulePoint( x , y , QR_TYPE_LIGHT );
			}
		}

		return true;
	}

	//{ԏo
	unsigned char ucDataBits[15 + 1] = "000000000000000";

	//G[̎擾

	unsigned char ucTypeData = (unsigned char)m_iQRMaskPattern & 0x07;
	ucTypeData |= (((c_typeMatrix + m_iModelTypeIndex)->m_iCollectLevel) << 3) & 0x18;

	//bTypeBits̓TɃZbg
	for( int i=0 ; i<5 ; i++ )
	{
		unsigned char ucMask = GetBitMask( 4 - i );
		ucDataBits[i] = ( (ucMask & ucTypeData) != 0 )?('1'):('0');
	}

	unsigned char ucCoefficients[] = "10100110111";

	bool bRet = false;

	//G[rbgǉ
	bRet = GetExpressionMod( ucDataBits , 5 , ucCoefficients , ucDataBits + 5 , 10 );

	//}XNp^[̓Kp
	unsigned char ucMask[] = "101010000010010";

	for( int i=0 ; i<15 ; i++ )
	{
		if( ucDataBits[i] == ucMask[i] )
		{
			ucDataBits[i] = '0';
		}
		else
		{
			ucDataBits[i] = '1';
		}
	}

	//o͏
	
	//㉡
	int iTemp = 0;
	for( int i = 0 ; i < 8 ; i++,iTemp++ )
	{
		//^C~Op^[ɏdȂꍇ
		if( i == 6 )
		{
			iTemp++;
		}

		int iPointData = QR_TYPE_DARK;
		if( ucDataBits[i] == '0' )
		{
			iPointData = QR_TYPE_LIGHT;
		}
		SetModulePoint( iTemp , 8 , iPointData );
	}

	//c
	iTemp = 0;
	for( int i = 0 ; i < 7 ; i++,iTemp++ )
	{
		//^C~Op^[ɏdȂꍇ
		if( i == 6 )
		{
			iTemp++;
		}

		int iPointData = QR_TYPE_DARK;
		if( ucDataBits[14-i] == '0' )
		{
			iPointData = QR_TYPE_LIGHT;
		}
		SetModulePoint( 8 , iTemp , iPointData );
	}

	//
	for( int i = 0 ; i < 8 ; i++ )
	{
		int iPointData = QR_TYPE_DARK;
		if( i != 7 && ucDataBits[i] == '0' )
		{
			iPointData = QR_TYPE_LIGHT;
		}

		SetModulePoint( 8 , m_iModuleNumber - i - 1 , iPointData );
	}

	//E
	for( int i = 0 ; i < 8 ; i++ )
	{
		int iPointData = QR_TYPE_DARK;
		if( ucDataBits[14-i] == '0' )
		{
			iPointData = QR_TYPE_LIGHT;
		}

		SetModulePoint( m_iModuleNumber - i - 1 , 8 , iPointData );
	}

	return bRet;
}

///////////////////////////////////////////////////////////
///		rbg̏]߂
///		puszCoefficients̒iErrorLength+1ƂȂ
///////////////////////////////////////////////////////////
bool CQRMatrix::GetExpressionMod( unsigned char *puszBaseData , int iBaseDataLength,
						   unsigned char *puszCoefficients ,
						   unsigned char *puszError , int iErrorLength )
{
	bool bRet = true;
	int iCoefficientsLength = iErrorLength + 1;

	size_t sizeDataLength = iBaseDataLength + iCoefficientsLength + 1;
	unsigned char * puszData = new unsigned char[sizeDataLength];

	//f[^
	memset( puszData , '0' , sizeDataLength - 1 );
	puszData[sizeDataLength - 1] = '\0';

#ifdef WIN32
	memcpy_s( puszData , sizeDataLength , puszBaseData , iBaseDataLength );
#else
	memcpy( puszData , puszBaseData , iBaseDataLength );
#endif

	for( int iKeta = 0 ; iKeta < iBaseDataLength ; iKeta++ )
	{
		if( puszData[iKeta] == '0' )
		{
			continue;
		}

		for( int i=0 ; i<iCoefficientsLength ; i++ )
		{
			if( puszData[iKeta + i] == puszCoefficients[i] )
			{
				puszData[iKeta + i] = '0';
			}
			else
			{
				puszData[iKeta + i] = '1';
			}
		}
	}

#ifdef WIN32
	memcpy_s( puszError , iErrorLength , puszData + iBaseDataLength , iErrorLength );
#else
	memcpy( puszError , puszData + iBaseDataLength , iErrorLength );
#endif

	delete[] puszData;

	return bRet;
}

///////////////////////////////////////////////////////////
///		^ԏ
///////////////////////////////////////////////////////////
bool CQRMatrix::SetModelInformation( int & iErrorType )
{
	//^ԏ7^̏ꍇi^ԏLqȂj
	if( (c_typeMatrix + m_iModelTypeIndex)->m_iModelNumber < 7 )
	{
		return true;
	}

	unsigned char ucDataBits[18 + 1] = "000000000000000000";
	unsigned char ucModel = (unsigned char)((c_typeMatrix + m_iModelTypeIndex)->m_iModelNumber) & 0x3f;

	for( int i=0 ; i<6 ; i++ )
	{
		unsigned char ucMask = GetBitMask( 5 - i );
		if( ucModel & ucMask )
		{
			ucDataBits[i] = '1';
		}
	}

	unsigned char ucCoefficients[] = "1111100100101";
	bool bRet = GetExpressionMod( ucDataBits , 6 , ucCoefficients , ucDataBits + 6 , 12 );

	//f[^̏o
	for( int i=0 ; i<6 ; i++ )
	{
		for( int j=0 ; j<3 ; j++ )
		{
			int iPointData = QR_MODEL_DARK;
			if( ucDataBits[(5 - i) * 3 + (2 - j)] == '0' )
			{
				iPointData = QR_MODEL_LIGHT;
			}

			//̌^ԃf[^o
			int x = i;
			int y = j + m_iModuleNumber - 11;
			SetModulePoint( x , y , iPointData );

			//Ě^ԃf[^o
			x = j + m_iModuleNumber - 11;
			y = i;
			SetModulePoint( x , y , iPointData );
		}
	}

	return true;
}

///////////////////////////////////////////////////////////
///		}XNp^[̓K̈̎擾
///		݂̃W[̈ŁAQR_UNKNOWNłȂ
///		}XNp^[K̈ƂăZbg
///////////////////////////////////////////////////////////
bool CQRMatrix::CreateMaskPatternArea( int & iErrorType )
{
	for( int i=0 ; i<m_iModuleNumber ; i++ )
	{
		for( int j=0 ; j<m_iModuleNumber ; j++ )
		{
			if( m_ppucQRMatrix[i][j] == QR_UNKNOWN )
			{
				m_ppucMaskMatrix[i][j] = QR_MASK_LIGHT;
			}
		}
	}

	return true;
}

///////////////////////////////////////////////////////////
///		w肳ꂽCfbNX̃rbg擾
///		߂lFONF0x01
///				OFFF0x00
///////////////////////////////////////////////////////////
unsigned char CQRMatrix::GetDataBit( int iBitIndex , bool bDataRequest )
{
	int iByteIndex = iBitIndex / 8;

	//f[^̃CfbNXl̏ꍇ
	if( iByteIndex < 0 )
	{
		return (unsigned char)0x00;
	}

	//}XN̎擾
	unsigned char ucMask = GetBitMask( 7 - iBitIndex % 8 );

	//f[^NGXgĂꍇ
	if( bDataRequest )
	{
		if( iByteIndex < m_iDataSizeNumber && m_pucData[iByteIndex] & ucMask )
		{
			return (unsigned char)0x01;
		}
	}
	//NGXgĂꍇ
	else
	{
		if( iByteIndex < m_iErrorSizeNumber && m_pucError[iByteIndex] & ucMask )
		{
			return (unsigned char)0x01;
		}
	}

	return (unsigned char)0x00;
}

///////////////////////////////////////////////////////////
///		f[^rbgp^[o͂
///////////////////////////////////////////////////////////
bool CQRMatrix::OutputDataBits( int & iErrorType )
{
	int iColumnIndex = m_iModuleNumber - 1;
	int iRowIndex    = m_iModuleNumber - 1;

	bool bDirectionUp = true;
	bool bFirstColumn = true;

	int iRSBlockNumber = (c_typeMatrix + m_iModelTypeIndex)->m_iRSBlockNumber;
	int iDataByteNumber = (c_typeMatrix + m_iModelTypeIndex)->m_iDataBitNumber / 8;
	int iBitIndex = 0;
	int iFirstCount , iSecondCount;
	int iFirstDataNumber , iSecondDataNumber;
	
	SeparateTotalNumber( iRSBlockNumber , iDataByteNumber 
						, iFirstCount , iFirstDataNumber 
						, iSecondCount , iSecondDataNumber );

	bool bRet = true;

	//ubNƂ̃f[^[v
	for( int iBlockIndex=0 ; iBlockIndex<iFirstDataNumber+1 ; iBlockIndex++ )
	{
		//ubN̏o
		for( int iSeq = 0 ; iSeq < iRSBlockNumber ; iSeq++ )
		{
			int iByteIndex = 0;

			//t@[XgJEg̏ꍇ
			if( iSeq < iFirstCount )
			{
				iByteIndex = iFirstDataNumber * iSeq;

				//ubNCfbNXf[^̈傫ꍇ
				if( iBlockIndex >= iFirstDataNumber )
				{
					continue;
				}
			}
			else
			{
				iByteIndex = iFirstDataNumber * iFirstCount + (iSeq - iFirstCount) * iSecondDataNumber;
			}

			iByteIndex += iBlockIndex;
			iBitIndex = iByteIndex * 8;

			for( int i=0 ; i<8 ; i++ )
			{
				GetNextDataPutPoint( iColumnIndex , iRowIndex , bFirstColumn , bDirectionUp );

				if( GetDataBit( iBitIndex , true ) )
				{
					bRet = SetModulePoint( iColumnIndex , iRowIndex , QR_DATA_DARK );
				}
				else
				{
					bRet = SetModulePoint( iColumnIndex , iRowIndex , QR_DATA_LIGHT );
				}

				iBitIndex++;
			}
		}
	}

	int iErrorByteNumber = ( (c_typeMatrix + m_iModelTypeIndex)->m_iTotalBits - 
							 (c_typeMatrix + m_iModelTypeIndex)->m_iDataBitNumber) / 8 / iRSBlockNumber;
	//ubNƂ̃f[^̃[v
	for( int iBlockIndex=0 ; iBlockIndex < iErrorByteNumber ; iBlockIndex++ )
	{
		//qrubNƂ̃f[^̃[v
		for( int iSeq = 0 ; iSeq < iRSBlockNumber ; iSeq++ )
		{
			int iByteIndex = iSeq * iErrorByteNumber + iBlockIndex;
			iBitIndex = iByteIndex * 8;

			for( int i=0 ; i<8 ; i++ )
			{
				GetNextDataPutPoint( iColumnIndex , iRowIndex , bFirstColumn , bDirectionUp );

				if( GetDataBit( iBitIndex , false ) )
				{
					bRet = SetModulePoint( iColumnIndex , iRowIndex , QR_COLLECT_DARK );
				}
				else
				{
					bRet = SetModulePoint( iColumnIndex , iRowIndex , QR_COLLECT_LIGHT );
				}

				iBitIndex++;
			}
		}
	}

	return bRet;
}

/////////////////////////////////////////////////////////////////////
///		̃f[^̃Zbg|Cg擾
/////////////////////////////////////////////////////////////////////
bool CQRMatrix::GetNextDataPutPoint( int & iColumnIndex , int & iRowIndex
									, bool & bFirstColumn
									, bool & bDirectionUp )
{
	//iColumnIndexiRowIndeẍʒu
	while( GetModulePoint( iColumnIndex , iRowIndex ) != QR_UNKNOWN )
	{
		bool bDirectionChanged = false;
		//Q̍ŏ̃f[^̏ꍇ
		if( bFirstColumn )
		{
			iColumnIndex--;
			bFirstColumn = false;
		}
		//cύXꍇ
		else
		{
			//Ɉړꍇ
			if( bDirectionUp )
			{
				//̍s
				iRowIndex--;

				//ŏi𒴂ꍇ
				if( iRowIndex < 0 )
				{
					iRowIndex = 0;
					iColumnIndex--;
					bDirectionUp = false;
				}
				else
				{
					iColumnIndex++;
				}
			}
			//Ɉړꍇ
			else
			{
				//̍s
				iRowIndex++;

				//ŉi𒴂ꍇ
				if( iRowIndex >= m_iModuleNumber )
				{
					iRowIndex = m_iModuleNumber - 1;
					iColumnIndex--;
					bDirectionUp = true;
				}
				else
				{
					iColumnIndex++;
				}
			}

			//i񂪁A^C~Op^[ic̗̏ꍇj
			if( iColumnIndex == 6 )
			{
				iColumnIndex = 5;
			}

			bFirstColumn = true;
		}
	}
	return true;
}

///////////////////////////////////////////////////////////
///		}XNp^[̌
///////////////////////////////////////////////////////////
bool CQRMatrix::DecideMaskPattern( int & iErrorType )
{
	int iMinLostPoint;
	int iMinQRMaskPattern;
	bool bRet = true;

	for( m_iQRMaskPattern = 0 ; m_iQRMaskPattern < 8 ; m_iQRMaskPattern++ )
	{
		int iLostPoint = 0;

		int iBlackPointNumber = 0;
		
		//񂲂Ƃ̃`FbN
		for( int iRowIndex = 0 ; iRowIndex < m_iModuleNumber ; iRowIndex++ )
		{
			int iSeqBlackModuleNumber = 0;
			int iSeqWhiteModuleNumber = 0;

			for( int iColumnIndex = 0 ; iColumnIndex < m_iModuleNumber ; iColumnIndex++ )
			{
				//Ã|Cg̏ꍇ
				if( IsDarkPoint( iColumnIndex , iRowIndex ) )
				{
					iSeqBlackModuleNumber++;
					iBlackPointNumber++;

					//A|Cg̃`FbN
					iLostPoint += GetLostPoint_SeqModule( iSeqWhiteModuleNumber );
					iSeqWhiteModuleNumber = 0;
				}
				//|Cg̏ꍇ
				else
				{
					iSeqWhiteModuleNumber++;

					//AÃ|Cg̃`FbN
					iLostPoint += GetLostPoint_SeqModule( iSeqBlackModuleNumber );
					iSeqBlackModuleNumber = 0;
				}

				//Q~QubN̎_JEg
				iLostPoint += GetLostPoint_ModuleBlock( iColumnIndex , iRowIndex );

				//1F1F3F1F1䗦̃`FbN
				iLostPoint += GetLostPoint_PositionDitect( iColumnIndex , iRowIndex );
			}

			//AÃ|Cg̃`FbN
			iLostPoint += GetLostPoint_SeqModule( iSeqBlackModuleNumber );
			//A|Cg̃`FbN
			iLostPoint += GetLostPoint_SeqModule( iSeqWhiteModuleNumber );
		}

		//Á^䗦
		iLostPoint += GetLostPoint_BlackWhiteRatio( iBlackPointNumber );

		//sƂ̃`FbN
		for( int iColumnIndex = 0 ; iColumnIndex < m_iModuleNumber ; iColumnIndex++ )
		{
			int iSeqBlackModuleNumber = 0;
			int iSeqWhiteModuleNumber = 0;

			//񂲂Ƃ̃`FbN
			for( int iRowIndex = 0 ; iRowIndex < m_iModuleNumber ; iRowIndex++ )
			{
				//Ã|Cg̏ꍇ
				if( IsDarkPoint( iColumnIndex , iRowIndex ) )
				{
					iSeqBlackModuleNumber++;

					//A|Cg̃`FbN
					iLostPoint += GetLostPoint_SeqModule( iSeqWhiteModuleNumber );
					iSeqWhiteModuleNumber = 0;
				}
				//|Cg̏ꍇ
				else
				{
					iSeqWhiteModuleNumber++;
					
					//AÃ|Cg̃`FbN
					iLostPoint += GetLostPoint_SeqModule( iSeqBlackModuleNumber );
					iSeqBlackModuleNumber = 0;
				}
			}

			//AÃ|Cg̃`FbN
			iLostPoint += GetLostPoint_SeqModule( iSeqBlackModuleNumber );
			//A|Cg̃`FbN
			iLostPoint += GetLostPoint_SeqModule( iSeqWhiteModuleNumber );
		}

		//Ԏ_̏Ȃ}XNp^[̎擾
		if( m_iQRMaskPattern == 0 )
		{
			iMinLostPoint = iLostPoint;
			iMinQRMaskPattern = 0;
		}
		else if( iMinLostPoint > iLostPoint )
		{
			iMinLostPoint = iLostPoint;
			iMinQRMaskPattern = m_iQRMaskPattern;
		}
	}

	m_iQRMaskPattern = iMinQRMaskPattern;

	return bRet;
}

///////////////////////////////////////////////////////////
///		_̃JEgiAÁEj
///////////////////////////////////////////////////////////
int CQRMatrix::GetLostPoint_SeqModule( int iSeqBlock )
{
	if( iSeqBlock < 5 )
	{
		return 0;
	}

	return (iSeqBlock - 5) + 3;
}

///////////////////////////////////////////////////////////
///		_̃JEgiwEsJn_ƂQ~QubNj
///////////////////////////////////////////////////////////
int CQRMatrix::GetLostPoint_ModuleBlock( int iColumnIndex , int iRowIndex )
{
	if( iColumnIndex >= m_iModuleNumber - 1 || iRowIndex >= m_iModuleNumber - 1 )
	{
		return 0;
	}

	bool bDark = IsDarkPoint( iColumnIndex , iRowIndex );
	if( bDark == IsDarkPoint( iColumnIndex + 1 , iRowIndex     ) &&
		bDark == IsDarkPoint( iColumnIndex     , iRowIndex + 1 ) &&
		bDark == IsDarkPoint( iColumnIndex + 1 , iRowIndex + 1 ) )
	{
		return 3;
	}
	return 0;
}

/////////////////////////////////////////////////////////////
///		_̃JEgiwʒu1F1F3F1F1̍s`FbN
///						}b`ꍇɂ͑O̖`FbNj
/////////////////////////////////////////////////////////////
int CQRMatrix::GetLostPoint_PositionDitect( int iColumnIndex , int iRowIndex )
{
	bool bDirectionColumn = true;
	int iLostPoint = 0;

	//c[v
	for( int i=0 ; i<2 ; i++,bDirectionColumn = false )
	{
		//ɂVhbgȏ̃f[^݂Ȃꍇ
		if( (bDirectionColumn && iColumnIndex >= m_iModuleNumber - 7) ||
			(!bDirectionColumn && iRowIndex >= m_iModuleNumber - 7 ) )
		{
			continue;
		}

		//1F1F3F1F1̃`FbN
		bool bMatched = true;

		for( int iIndex = 0 ; iIndex<7 ; iIndex++ )
		{
			bool bNeedDark = false;
			if( iIndex == 3 || iIndex % 2 == 0 )
			{
				bNeedDark = true;
			}

			if( IsDarkPoint( iColumnIndex + ((bDirectionColumn)?(iIndex):(0))
							,iRowIndex    + ((bDirectionColumn)?(0):(iIndex)) ) != bNeedDark )
			{
				bMatched = false;
				break;
			}
		}

		//1F1F3F1F1݂Ȃꍇ
		if( !bMatched )
		{
			continue;
		}

		//ÕXy[XJEg
		if( ( bDirectionColumn && iColumnIndex > 3) || 
			(!bDirectionColumn && iRowIndex > 3 ) )
		{
			bMatched = true;

			for( int iIndex = 0 ; iIndex < 4 ; iIndex++ )
			{
				int iPlusCount = -iIndex - 1;
				if( IsDarkPoint( iColumnIndex + ((bDirectionColumn)?(iPlusCount):(0))
								,iRowIndex    + ((bDirectionColumn)?(0):(iPlusCount)) ) )
				{
					bMatched = false;
					break;
				}
			}

			//OS̏ꍇ
			if( bMatched )
			{
				iLostPoint += 40;
				continue;
			}
		}

		//̃Xy[XJEg
		if( ( bDirectionColumn && iColumnIndex < m_iModuleNumber - 7 - 4 ) ||
			(!bDirectionColumn && iRowIndex    < m_iModuleNumber - 7 - 4 ) )
		{
			bMatched = true;

			for( int iIndex = 0 ; iIndex < 4 ; iIndex++ )
			{
				int iPlusCount = iIndex + 7 + i;
				if( IsDarkPoint( iColumnIndex + ((bDirectionColumn)?(iPlusCount):(0))
								,iRowIndex    + ((bDirectionColumn)?(0):(iPlusCount)) ) )
				{
					bMatched = false;
					break;
				}
			}

			//S̏ꍇ
			if( bMatched )
			{
				iLostPoint += 40;
			}
		}

	}
	return iLostPoint;
}

///////////////////////////////////////////////////////////
///		_̃JEgiÁE䗦j
///////////////////////////////////////////////////////////
int CQRMatrix::GetLostPoint_BlackWhiteRatio( int iBlackPoint )
{
	int iAllNumber = m_iModuleNumber * m_iModuleNumber;
	int iMod = iAllNumber / 2 - iBlackPoint;
	if( iMod < 0 )
	{
		iMod = -iMod;
	}

	return ( ( (iMod * 100 + iAllNumber - 1) / iAllNumber) / 5 ) * 10;
}
