
#include <stdlib.h>
#include <string.h>

#include "QRReadSolomon.h"
#include "QRReadSolomonInformation.h"

CQRReadSolomon::CQRReadSolomon()
{
	m_pNumberAlpha = new SAlphaNumber[ 256 ];
	for( int i=0 ; i<256 ; i++ )
	{
		m_pNumberAlpha[i] = c_alphaNumber[i];
	}
	qsort( m_pNumberAlpha , 256 , sizeof(SAlphaNumber) , SortCompareNumberAlpha );

	(m_pNumberAlpha  )->m_sNumber = 0;
	(m_pNumberAlpha  )->m_sAlpha = 0;

	(m_pNumberAlpha+1)->m_sNumber = 1;
	(m_pNumberAlpha+1)->m_sAlpha = 0;

}

CQRReadSolomon::~CQRReadSolomon()
{
	delete[] m_pNumberAlpha;
}

int CQRReadSolomon::SortCompareNumberAlpha( const void * pcParam1 , const void * pcParam2 )
{
	SAlphaNumber * pAlphaNumber1 = (SAlphaNumber*)pcParam1;
	SAlphaNumber * pAlphaNumber2 = (SAlphaNumber*)pcParam2;

	return pAlphaNumber1->m_sNumber - pAlphaNumber2->m_sNumber;
}

///////////////////////////////////////////////////////////
///		[h\̎s
///		F
///			pucBase		F	(IN)	R[hsf[^
///			iDataLength	F	(IN)	R[hsf[^
///			pucCode		F	(OUT)	R[hʂ̊i[pz
///			iCodeLength	F	(IN)	R[hʂ̊i[pzvfR[hꐔ
///////////////////////////////////////////////////////////
bool CQRReadSolomon::CodeTo( unsigned char * pucBase , int iBaseLength
							,unsigned char * pucCode , int iCodeLength )
{
	bool bRet = true;

	//̌W擾
	unsigned char * pucCoefficients = GetExpressionCoefficients( iCodeLength );

	if( pucCoefficients == NULL )
	{
		return false;
	}

	//x[Xf[^̕쐬
	int iDataLength = iBaseLength + iCodeLength;
	unsigned char * pucData = new unsigned char[iDataLength];
	memset( pucData , 0 , iDataLength );
#ifdef WIN32
	memcpy_s( pucData , iDataLength , pucBase , iBaseLength );
#else
	memcpy( pucData , pucBase , iBaseLength );
#endif

	int iDataLast = iDataLength - iCodeLength;
	for( int iDataIndex = 0; iDataIndex < iDataLast ; iDataIndex++ )
	{
		//f[^W
		short sKeisu = (short)pucData[iDataIndex];

		//ΏۂƂȂ錅0̏ꍇ
		if( !sKeisu )
		{
			continue;
		}
		
		//Wɑ΂At@̗ݏ擾
		short sAlpha = (m_pNumberAlpha + sKeisu )->m_sAlpha ;

		//̌WXgɃݏZiݏ擯m̊|Z搔̑Zj
		for( int i=0 ; i<iCodeLength+1 ; i++ )
		{
			short sTemp = (short)pucCoefficients[i] + sAlpha;

			//O255==1A𗎂Ƃ
			while( sTemp > 255 )
			{
				sTemp -= 255;
			}

			//e̘_a擾Af[^rbg𖄂߂
			pucData[iDataIndex + i] = (pucData[iDataIndex + i]) ^ ((c_alphaNumber + sTemp)->m_sNumber);
		}

	}

	//]f[^zɃZbg
	for( int i=0 ; i<iCodeLength ; i++ )
	{
		int iDataIndex = iDataLength - i - 1;
		int iCodeIndex = iCodeLength - i - 1;

		pucCode[iCodeIndex] = pucData[iDataIndex];
	}

	//̌WXg폜
	delete[] pucCoefficients;
	delete[] pucData;

	return bRet;
}

///////////////////////////////////////////////////////////
///		̌W擾
///		F
///			iCodeLength	F	(IN)	R[hꐔ
///		߂lF
///			W̗ݏ惊XgipucCoefficientsj
///			x^iCodeLength      *^(pucCoefficients[0])
///			+ x^(iCodeLength-1)*^(pucCoefficients[1])
///			+ x^(iCodeLength-2)*^(pucCoefficients[2])
///			+ cccc
///			+ x                *^(pucCoefficients[iCodeLength-1])
///			+ 1                *^(pucCoefficients[iCodeLength  ])
///
///			i^j͗ݏӖBxorł͂Ȃ̂ŔÔ
///////////////////////////////////////////////////////////
unsigned char * CQRReadSolomon::GetExpressionCoefficients( int iCodeLength )
{
	int iExpressionIndex = 0;
	for( ; (c_readSolomonExpression + iExpressionIndex)->m_iCodeNumber != iCodeLength && 
		   (c_readSolomonExpression + iExpressionIndex)->m_iCodeNumber >= 0
		 ; iExpressionIndex++ );

	//̃XgɎw̃R[hꐔf[^݂Ȃꍇ
	if( (c_readSolomonExpression + iExpressionIndex)->m_iCodeNumber != iCodeLength )
	{
		return NULL;
	}

	unsigned char * pucCoefficients = new unsigned char[iCodeLength + 1];
	
	//Xg̃[v
	
	size_t sizeLength = strlen( (c_readSolomonExpression + iExpressionIndex)->m_pcszExpression) + 1;
	char * pszExpression = new char[sizeLength];
#ifdef WIN32
	strcpy_s( pszExpression , sizeLength , (c_readSolomonExpression + iExpressionIndex)->m_pcszExpression );
#else
	strcpy( pszExpression , (c_readSolomonExpression + iExpressionIndex)->m_pcszExpression );
#endif

	//J}؂PXgɕύX
	for( int i=0 ; i<(int)sizeLength ; i++ )
	{
		if( *(pszExpression + i) == ',' )
		{
			*(pszExpression + i) = '\0';
		}
	}

	int iIndex = 0;
	for( int iStringIndex = 0
		; pszExpression[iStringIndex] && iIndex < iCodeLength + 1
		; iStringIndex += (int)strlen( pszExpression + iStringIndex ) + 1, iIndex++ )
	{
		pucCoefficients[iIndex] = (unsigned char)atoi( pszExpression + iStringIndex);
	}

	delete[] pszExpression;

	return pucCoefficients;
}
